==> emf.doc_1.0.2_20030326_0335VL.zip: src/org/eclipse/emf/ecore/xmi/impl/XMLResourceImpl.java <==
/**
 * <copyright>
 *
 * Copyright (c) 2002 IBM Corporation and others.
 * All rights reserved.   This program and the accompanying materials
 * are made available under the terms of the Common Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/cpl-v10.html
 *
 * Contributors:
 *   IBM - Initial API and implementation
 *
 * </copyright>
 *
 * plugins/org.eclipse.emf.ecore.xmi/src/org/eclipse/emf/ecore/xmi/impl/XMLResourceImpl.java, emf.ecore.xmi, org.eclipse.102, 20030326_0335VL
 * @version 1.9 3/26/03
 */
package org.eclipse.emf.ecore.xmi.impl;

import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.util.Collections;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;
import java.util.TreeMap;

import org.eclipse.emf.common.util.URI;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.resource.impl.ResourceImpl;
import org.eclipse.emf.ecore.xmi.XMLHelper;
import org.eclipse.emf.ecore.xmi.XMLLoad;
import org.eclipse.emf.ecore.xmi.XMLResource;
import org.eclipse.emf.ecore.xmi.XMLSave;

/**
 * This class implements the XMLResource interface. It overloads the
 * doLoad method to invoke the XML deserializer rather than using the
 * default XMI loader.
 */
public class XMLResourceImpl extends ResourceImpl implements XMLResource
{
  /**
   * The map from {@link #getID ID} to {@link EObject}.
   * It is used to store IDs during a load or if the user
   * sets the ID of an object.
   */
  protected Map idToEObjectMap;

  /**
   * The map from {@link EObject} to {@link #getID ID}.
   * It is used to store IDs during a load or if the user
   * sets the ID of an object.
   */
  protected Map eObjectToIDMap;

  protected String encoding;
  protected boolean useZip;

  protected XMLLoad xmlLoad;
  protected XMLSaveImpl xmlSave;

  /**
   * Constructor for XMLResourceImpl.
   */
  public XMLResourceImpl()
  {
    super();
    init();
  }

  /**
   * Constructor for XMLResourceImpl.
   * @param uri
   */
  public XMLResourceImpl(URI uri)
  {
    super(uri);
    init();
  }

  protected void init()
  {
    encoding = "ASCII";
  }

  protected boolean useIDs()
  {
    return true;
  }

  public Map getDefaultSaveOptions()
  {
    if (defaultSaveOptions == null)
    {
      defaultSaveOptions = new HashMap();
    }
    return defaultSaveOptions;
  }

  public Map getDefaultLoadOptions()
  {
    if (defaultLoadOptions == null)
    {
      defaultLoadOptions = new HashMap();
    }
    return defaultLoadOptions;
  }

  protected XMLHelper createXMLHelper()
  {
    return new XMLHelperImpl(this);
  }

  protected XMLLoad createXMLLoad()
  {
    return new XMLLoadImpl(createXMLHelper());
  }

  protected XMLSave createXMLSave()
  {
    return new XMLSaveImpl(createXMLHelper());
  }

  public void doLoad(InputStream inputStream, Map options) throws IOException
  {
    XMLLoad xmlLoad = createXMLLoad();

    if (options == null)
    {
      options = Collections.EMPTY_MAP;
    }

    xmlLoad.load(this, inputStream, options);
  }

  public void doSave(OutputStream outputStream, Map options) throws IOException
  {
    XMLSave xmlSave = createXMLSave();

    if (options == null)
    {
      options = Collections.EMPTY_MAP;
    }

    xmlSave.save(this, outputStream, options);
  }

  public boolean useZip()
  {
    return useZip;
  }

  public void setUseZip(boolean useZip)
  {
    this.useZip = useZip;
  }

  public String getEncoding()
  {
    return encoding;
  }

  public void setEncoding(String encoding)
  {
    this.encoding = encoding;
  }

  public Map getIDToEObjectMap()
  {
    if (idToEObjectMap == null)
    {
      idToEObjectMap = new HashMap();
    }

    return idToEObjectMap;
  }

  public Map getEObjectToIDMap()
  {
    if (eObjectToIDMap == null)
    {
      eObjectToIDMap = new HashMap();
    }

    return eObjectToIDMap;
  }

  /*
   * Javadoc copied from interface
   */
  public String getID(EObject eObject)
  {
    if (eObjectToIDMap == null)
    {
      return null;
    }
    else
    {
      return (String) eObjectToIDMap.get(eObject);
    }
  }

  /**
   * Sets the ID of the object.
   * This default implementation will update the {@link #eObjectToIDMap}.
   * Clients may override it to set the ID as an actual attribute object the object.
   * @param eObject the object.
   * @param id the object's ID.
   */
  public void setID(EObject eObject, String id)
  {
    getEObjectToIDMap().put(eObject, id);
    getIDToEObjectMap().put(id, eObject);
  }

  /*
   * Javadoc copied from interface.
   */
  public String getURIFragment(EObject eObject)
  {
    String id = getID(eObject);

    if (id != null)
    {
      return id;
    }
    else
    {
      return super.getURIFragment(eObject);
    }
  }

  protected EObject getEObjectByID(String id)
  {
    if (idToEObjectMap != null)
    {
      EObject eObject = (EObject) idToEObjectMap.get(id);
      if (eObject != null)
      {
        return eObject;
      }
    }

   return super.getEObjectByID(id);
  }

  protected boolean isPath(String uriFragment)
  {
    return uriFragment.startsWith("/");
  }

  /*
   * Javadoc copied from interface.
   */
  public void attached(EObject eObject)
  {
    if (useIDs())
    {

      if (modificationTrackingAdapter != null)
      {
        eObject.eAdapters().add(modificationTrackingAdapter);
      }

      String id = getID(eObject);

      if (id != null)
      {
        getIDToEObjectMap().put(id, eObject);
      }

      for (Iterator tree = eObject.eAllContents(); tree.hasNext(); )
      {
        EObject child = (EObject)tree.next();
        String childID = getID(child);
        if (childID != null)
        {
          getIDToEObjectMap().put(childID, child);
        }
        if (modificationTrackingAdapter != null)
        {
          child.eAdapters().add(modificationTrackingAdapter);
        }
      }
    }
    else
    {
      super.attached(eObject);
    }
  }

  /*
   * Javadoc copied from interface.
   */
  public void detached(EObject eObject)
  {
    if (useIDs())
    {
      eObject.eAdapters().remove(modificationTrackingAdapter);

      if (idToEObjectMap != null && eObjectToIDMap != null)
      {
        idToEObjectMap.remove(eObjectToIDMap.remove(eObject));
      }

      for (Iterator tree = eObject.eAllContents(); tree.hasNext(); )
      {
        EObject child = (EObject)tree.next();

        if (idToEObjectMap != null && eObjectToIDMap != null)
        {
          idToEObjectMap.remove(eObjectToIDMap.remove(child));
        }

        child.eAdapters().remove(modificationTrackingAdapter);
      }
    }
    else
    {
      super.detached(eObject);
    }
  }

  /**
   * Does all the work of unloading the resource. It calls doUnload in
   * ResourceImpl, then it clears {@link #idToEObjectMap} and {@link #eObjectToIDMap} as necessary.
   */
  protected void doUnload()
  {
    super.doUnload();

    if (idToEObjectMap != null)
    {
      idToEObjectMap.clear();
    }

    if (eObjectToIDMap != null)
    {
      eObjectToIDMap.clear();
    }
  }

  /*
   * Javadoc copied from interface.
   */
  public void changed(EObject eObject, String oldID)
  {
    idToEObjectMap.remove(oldID);
    idToEObjectMap.put(getID(eObject), eObject);
  }

  /**
   * Returns a string representation of the {@link #idToEObjectMap ID} map.
   * @return a string representation of the ID map.
   */
  public String toKeyString()
  {
    StringBuffer result = new StringBuffer("Key type: ");
    result.append(getClass().toString());
    if (idToEObjectMap != null)
    {
      TreeMap tree = new TreeMap();
      for (Iterator i = idToEObjectMap.keySet().iterator(); i.hasNext(); )
      {
        Object key = i.next();
        if (key != null)
        {
          tree.put(key.toString(), key);
        }
      }

      // add the key/value pairs to the output string
      for (Iterator i = tree.values().iterator(); i.hasNext(); )
      {
        Object key = i.next();
        Object value = idToEObjectMap.get(key);
        result.append("\r\n\t[Key=" + key + ", Value=" + value + "]");
      }
    }
    return result.toString();
  }
}

==> emf.doc_1.1.0_20030620_1105VL.zip: src/org/eclipse/emf/ecore/xmi/impl/XMLResourceImpl.java <==
/**
 * <copyright>
 *
 * Copyright (c) 2002 IBM Corporation and others.
 * All rights reserved.   This program and the accompanying materials
 * are made available under the terms of the Common Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/cpl-v10.html
 *
 * Contributors:
 *   IBM - Initial API and implementation
 *
 * </copyright>
 *
 * plugins/org.eclipse.emf.ecore.xmi/src/org/eclipse/emf/ecore/xmi/impl/XMLResourceImpl.java, emf.ecore.xmi, org.eclipse.dev, 20030620_1105VL
 * @version 1.10 6/20/03
 */
package org.eclipse.emf.ecore.xmi.impl;

import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.util.Collections;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;
import java.util.TreeMap;

import org.eclipse.emf.common.util.URI;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.resource.impl.ResourceImpl;
import org.eclipse.emf.ecore.xmi.XMLHelper;
import org.eclipse.emf.ecore.xmi.XMLLoad;
import org.eclipse.emf.ecore.xmi.XMLResource;
import org.eclipse.emf.ecore.xmi.XMLSave;

/**
 * This class implements the XMLResource interface. It overloads the
 * doLoad method to invoke the XML deserializer rather than using the
 * default XMI loader.
 */
public class XMLResourceImpl extends ResourceImpl implements XMLResource
{
  /**
   * The map from {@link #getID ID} to {@link EObject}.
   * It is used to store IDs during a load or if the user
   * sets the ID of an object.
   */
  protected Map idToEObjectMap;

  /**
   * The map from {@link EObject} to {@link #getID ID}.
   * It is used to store IDs during a load or if the user
   * sets the ID of an object.
   */
  protected Map eObjectToIDMap;

  protected String encoding;
  protected boolean useZip;

  protected XMLLoad xmlLoad;
  protected XMLSaveImpl xmlSave;

  /**
   * Constructor for XMLResourceImpl.
   */
  public XMLResourceImpl()
  {
    super();
    init();
  }

  /**
   * Constructor for XMLResourceImpl.
   * @param uri
   */
  public XMLResourceImpl(URI uri)
  {
    super(uri);
    init();
  }

  protected void init()
  {
    encoding = "ASCII";
  }

  protected boolean useIDs()
  {
    return true;
  }

  public Map getDefaultSaveOptions()
  {
    if (defaultSaveOptions == null)
    {
      defaultSaveOptions = new HashMap();
    }
    return defaultSaveOptions;
  }

  public Map getDefaultLoadOptions()
  {
    if (defaultLoadOptions == null)
    {
      defaultLoadOptions = new HashMap();
    }
    return defaultLoadOptions;
  }

  protected XMLHelper createXMLHelper()
  {
    return new XMLHelperImpl(this);
  }

  protected XMLLoad createXMLLoad()
  {
    return new XMLLoadImpl(createXMLHelper());
  }

  protected XMLSave createXMLSave()
  {
    return new XMLSaveImpl(createXMLHelper());
  }

  public void doLoad(InputStream inputStream, Map options) throws IOException
  {
    XMLLoad xmlLoad = createXMLLoad();

    if (options == null)
    {
      options = Collections.EMPTY_MAP;
    }

    xmlLoad.load(this, inputStream, options);
  }

  public void doSave(OutputStream outputStream, Map options) throws IOException
  {
    XMLSave xmlSave = createXMLSave();

    if (options == null)
    {
      options = Collections.EMPTY_MAP;
    }

    xmlSave.save(this, outputStream, options);
  }

  public boolean useZip()
  {
    return useZip;
  }

  public void setUseZip(boolean useZip)
  {
    this.useZip = useZip;
  }

  public String getEncoding()
  {
    return encoding;
  }

  public void setEncoding(String encoding)
  {
    this.encoding = encoding;
  }

  public Map getIDToEObjectMap()
  {
    if (idToEObjectMap == null)
    {
      idToEObjectMap = new HashMap();
    }

    return idToEObjectMap;
  }

  public Map getEObjectToIDMap()
  {
    if (eObjectToIDMap == null)
    {
      eObjectToIDMap = new HashMap();
    }

    return eObjectToIDMap;
  }

  /*
   * Javadoc copied from interface
   */
  public String getID(EObject eObject)
  {
    if (eObjectToIDMap == null)
    {
      return null;
    }
    else
    {
      return (String) eObjectToIDMap.get(eObject);
    }
  }

  /**
   * Sets the ID of the object.
   * This default implementation will update the {@link #eObjectToIDMap}.
   * Clients may override it to set the ID as an actual attribute object the object.
   * @param eObject the object.
   * @param id the object's ID.
   */
  public void setID(EObject eObject, String id)
  {
    Object oldID = getEObjectToIDMap().put(eObject, id);
    if (oldID != null)
    {
      getIDToEObjectMap().remove(oldID);
    }
    getIDToEObjectMap().put(id, eObject);
  }

  /*
   * Javadoc copied from interface.
   */
  public String getURIFragment(EObject eObject)
  {
    String id = getID(eObject);

    if (id != null)
    {
      return id;
    }
    else
    {
      return super.getURIFragment(eObject);
    }
  }

  protected EObject getEObjectByID(String id)
  {
    if (idToEObjectMap != null)
    {
      EObject eObject = (EObject) idToEObjectMap.get(id);
      if (eObject != null)
      {
        return eObject;
      }
    }

   return super.getEObjectByID(id);
  }

  protected boolean isPath(String uriFragment)
  {
    return uriFragment.startsWith("/");
  }

  /*
   * Javadoc copied from interface.
   */
  public void attached(EObject eObject)
  {
    if (useIDs())
    {

      if (modificationTrackingAdapter != null)
      {
        eObject.eAdapters().add(modificationTrackingAdapter);
      }

      String id = getID(eObject);

      if (id != null)
      {
        getIDToEObjectMap().put(id, eObject);
      }

      for (Iterator tree = eObject.eAllContents(); tree.hasNext(); )
      {
        EObject child = (EObject)tree.next();
        String childID = getID(child);
        if (childID != null)
        {
          getIDToEObjectMap().put(childID, child);
        }
        if (modificationTrackingAdapter != null)
        {
          child.eAdapters().add(modificationTrackingAdapter);
        }
      }
    }
    else
    {
      super.attached(eObject);
    }
  }

  /*
   * Javadoc copied from interface.
   */
  public void detached(EObject eObject)
  {
    if (useIDs())
    {
      eObject.eAdapters().remove(modificationTrackingAdapter);

      if (idToEObjectMap != null && eObjectToIDMap != null)
      {
        idToEObjectMap.remove(eObjectToIDMap.remove(eObject));
      }

      for (Iterator tree = eObject.eAllContents(); tree.hasNext(); )
      {
        EObject child = (EObject)tree.next();

        if (idToEObjectMap != null && eObjectToIDMap != null)
        {
          idToEObjectMap.remove(eObjectToIDMap.remove(child));
        }

        child.eAdapters().remove(modificationTrackingAdapter);
      }
    }
    else
    {
      super.detached(eObject);
    }
  }

  /**
   * Does all the work of unloading the resource. It calls doUnload in
   * ResourceImpl, then it clears {@link #idToEObjectMap} and {@link #eObjectToIDMap} as necessary.
   */
  protected void doUnload()
  {
    super.doUnload();

    if (idToEObjectMap != null)
    {
      idToEObjectMap.clear();
    }

    if (eObjectToIDMap != null)
    {
      eObjectToIDMap.clear();
    }
  }

  /**
   * Returns a string representation of the {@link #idToEObjectMap ID} map.
   * @return a string representation of the ID map.
   */
  public String toKeyString()
  {
    StringBuffer result = new StringBuffer("Key type: ");
    result.append(getClass().toString());
    if (idToEObjectMap != null)
    {
      TreeMap tree = new TreeMap();
      for (Iterator i = idToEObjectMap.keySet().iterator(); i.hasNext(); )
      {
        Object key = i.next();
        if (key != null)
        {
          tree.put(key.toString(), key);
        }
      }

      // add the key/value pairs to the output string
      for (Iterator i = tree.values().iterator(); i.hasNext(); )
      {
        Object key = i.next();
        Object value = idToEObjectMap.get(key);
        result.append("\r\n\t[Key=" + key + ", Value=" + value + "]");
      }
    }
    return result.toString();
  }
}
